/*******************************************************************************
 * DERMI: A DHT-based Peer-to-Peer Middleware for Distributed Components.
 * Copyright (C) 2003-2004 Carles Pairot
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ******************************************************************************/
package dermi.samples.listener;

import dermi.*;
import dermi.exception.*;
import dermi.registry.FileRegistry;

import java.util.*;
import java.io.*;

/**
 * AUTOMATICALLY GENERATED by DERMIC TOOL
 * This class implements the stub class for Sprite object
 * @author Carles Pairot   <cpairot@etse.urv.es>
 */
public  class SpriteStub extends EventServer implements RemoteEventListener, Sprite, ERemote {
  private Hashtable subs = new Hashtable();
  private boolean getXResultArrived = false;
  private boolean setXResultArrived = false;
  private boolean exception = false;
  private Object getXResult;
  private Object setXResult;
  private String method;
  private int timeout;
  private int listenerCount = 0;
  private Vector listeners = new Vector();
  private HashSet receivedEvents = new HashSet();

  /**
   * Constructor: entry point to the stub class
   * @param env Properties Environment properties
   * @throws RemoteException If something goes wrong ;-)
   */
  public SpriteStub (Properties env) throws RemoteException {
    super (env);
    String muid = env.getProperty (Context.CLASS_ID);
    uid = muid;
    className = this.getClass().getName().replaceAll ("Stub", "");
    subs.put ("OBJ", uid + ":RESULT");
    try {
      conn.subscribe (this, subs);
    } catch (SessionException e) {
      throw new ConnectionException (e.getMessage());
    }
  }

  /**
   * Constructor: entry point to the stub class
   * @param env Properties Environment properties
   * @param stub String Stub name
   * @throws RemoteException If something goes wrong ;-)
   */
  public SpriteStub (Properties env, String stub) throws RemoteException {
    super (env);
    String muid = env.getProperty (Context.CLASS_ID);
    uid = muid;
    className = this.getClass().getName().replaceAll ("Stub", "");
    subs.put ("OBJ", uid + ":RESULT");
  }

  /**
   * Method invoked each time the stub received an event
   * @param evt Hashtable Information about the received event
   */
  public void eventArrived (Hashtable evt) {
    method = (String) evt.get ("METHOD");
    if (!receivedEvents.contains (method + (String) evt.get ("TIMESTAMP"))) {
      String type;
      if (method.equals ("getX")) {
        getXResult = evt.get ("object");
        type = (String) evt.get ("TYPE");
        if (type.equals ("EXCEPTION")) {
          exception = true;
        }
        getXResultArrived = true;
      }
      else if (method.equals ("setX")) {
        setXResult = evt.get ("object");
        type = (String) evt.get ("TYPE");
        if (type.equals ("EXCEPTION")) {
          exception = true;
        }
        setXResultArrived = true;
      }
    }
    receivedEvents.add (method + (String) evt.get ("TIMESTAMP"));
  }

  /**
   * Generated stub method: getX
   * @throws RemoteException If something goes wrong ;-)
   * @return String Return result
   */
  public String getX() throws RemoteException {
    // Set timestamp for this event and its possible retransmissions
    String tstamp = "" + System.currentTimeMillis();

    for (int retransmission = 0; retransmission < Context.RETRANSMISSIONS;) {
      Hashtable evt = new Hashtable();
      evt.put ("OBJECT", uid);
      evt.put ("TYPE", "DISPATCH");
      evt.put ("METHOD", "getX");
      evt.put ("TIMESTAMP", tstamp);
      evt.put ("SOURCE_NHANDLE", conn.getLocalNodeHandle());
      getXResultArrived = false;
      dispatchEvent (evt);
      timeout = 0;

      while (!getXResultArrived && timeout < Context.TIMEOUT) {
        try {
          Thread.currentThread().sleep (Context.DELAY);
        }
        catch (InterruptedException ex) {
          ex.printStackTrace();
        }
        timeout++;
      }
      if (exception) {
        exception = false;
        RemoteException ex = (RemoteException) getXResult;
        throw ex;
      }
      else if (timeout == Context.TIMEOUT) {
        retransmission++;
        continue;
      }
      break;
    }
    if (timeout == Context.TIMEOUT) {
      throw new TimeoutException ("Object seems to be down.");
    }
    else {
      String res = (String) getXResult;
      return res;
    }
  }

  /**
   * Generated stub method: setX
   * @param p0 String Input parameter 0
   * @throws RemoteException If something goes wrong ;-)
   */
  public void setX (String p0) throws RemoteException {
    // Set timestamp for this event and its possible retransmissions
    String tstamp = "" + System.currentTimeMillis();

    for (int retransmission = 0; retransmission < Context.RETRANSMISSIONS;) {
      Hashtable evt = new Hashtable();
      evt.put ("OBJECT", uid);
      evt.put ("TYPE", "DISPATCH");
      evt.put ("METHOD", "setX");
      evt.put ("TIMESTAMP", tstamp);
      evt.put ("SOURCE_NHANDLE", conn.getLocalNodeHandle());
      setXResultArrived = false;
      Vector args = new Vector();
      args.add (p0);
      dispatchEvent (evt, args);
      timeout = 0;
      while (!setXResultArrived && timeout < Context.TIMEOUT) {
        try {
          Thread.currentThread().sleep (Context.DELAY);
        } catch (InterruptedException ex) {
          ex.printStackTrace();
        }
        timeout++;
      }
      if (exception) {
        exception = false;
        RemoteException ex = (RemoteException) setXResult;
        throw ex;
      } else if (timeout == Context.TIMEOUT) {
        retransmission++;
        continue;
      }
      break;
   }
    if (timeout == Context.TIMEOUT) {
      throw new TimeoutException ("Object seems to be down.");
    }
  }

  /**
   * Generated listener method: addSpriteListener
   * @param p0 SpriteListener Input parameter 0
   * @throws RemoteException If something goes wrong ;-)
   */
  public void addSpriteListener (SpriteListener p0) throws RemoteException {
    if (listenerCount == 0) {
      Hashtable lsubs = new Hashtable();
      lsubs.put ("OBJ", uid + ":RESULT:SpriteEvent");
      try {
        conn.subscribe (new RemoteEventListener() {
          public void eventArrived (Hashtable evt) {
            SpriteEvent result = (SpriteEvent) evt.get ("object");
            Vector clisteners = (Vector) listeners.clone();
            Iterator it = clisteners.iterator();
            while (it.hasNext()) {
              SpriteListener aListener = (SpriteListener) it.next();
              aListener.onEvent (result);
            }
          }
        }, lsubs);
      } catch (SessionException ex) {
        throw new ConnectionException ("Problem subscribing listener.");
      }
    }
    listeners.add (p0);
    listenerCount++;
  }

  /**
   * Generated listener method: removeSpriteListener
   * @param p0 SpriteListener Input parameter 0
   * @throws RemoteException If something goes wrong ;-)
   */
  public void removeSpriteListener (SpriteListener p0) throws RemoteException {
    listeners.remove (p0);
    listenerCount--;
    if (listenerCount == 0) {
      Hashtable lsubs = new Hashtable();
      lsubs.put ("OBJ", uid + ":RESULT:SpriteEvent");
      try {
        conn.unsubscribe (this, lsubs);
      }catch (SessionException ex) {
        throw new ConnectionException ("Problem unsubscribing listener.");
      }
    }
  }

  /**
   * Closes the stub
   * @throws RemoteException If something goes wrong ;-)
   */
  public void close() throws RemoteException {
    try {
      conn.unsubscribe (this, subs);
      dermi.util.Bus.close();
    } catch (SessionException e) {
      throw new ConnectionException(e.getMessage());
    }
  }

  /**
   * Initializes the stub
   * @throws RemoteException  If something goes wrong ;-)
   */
  public void init() throws RemoteException {
    conn = dermi.util.Bus.getBus (eref);
    try {
      conn.subscribe (this, subs);
    } catch (SessionException e) {
      throw new ConnectionException (e.getMessage());
    }
  }

  /**
   * Writes the current object to a stream
   * @param out ObjectOutputStream Data stream
   * @throws IOException  If something goes wrong ;-)
   */
  private void writeObject (ObjectOutputStream out) throws IOException {
    out.defaultWriteObject();
  }

  /**
   * Reads an object from a stream
   * @param in ObjectInputStream Data stream
   * @throws IOException  If something goes wrong ;-)
   * @throws ClassNotFoundException  If something goes wrong ;-)
   */
  private void readObject (ObjectInputStream in) throws IOException, ClassNotFoundException {
    try {
      in.defaultReadObject();
      init();
    } catch (RemoteException e) {
      e.printStackTrace();
    }
  }

  /**
   * Compares this object with another
   * @param obj Object The other object
   * @return boolean true if both equal
   */
  public boolean equals (Object obj) {
    ERemote ref = (ERemote) obj;
    return ref.getERef().equals (getERef());
  }

  /**
   * Returns a stub copy of this class
   * @throws RemoteException If something goes wrong ;-)
   * @return ERemote The returned stub
   */
  public ERemote copy() throws RemoteException {
    ERemote ref = FileRegistry.loadStub (getERef());
    return (ERemote) ref;
  }
}
