/*******************************************************************************
 * DERMI: A DHT-based Peer-to-Peer Middleware for Distributed Components.
 * Copyright (C) 2003-2004 Carles Pairot
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ******************************************************************************/
package dermi.samples.manycall;

import dermi.*;
import dermi.exception.*;
import dermi.registry.FileRegistry;

import java.io.*;
import java.util.*;

/**
 * AUTOMATICALLY GENERATED by DERMIC TOOL
 * This class implements the stub class for the Voting object
 * @author Carles Pairot   <cpairot@etse.urv.es>
 */
public class VotingStub extends EventServer implements RemoteEventListener, Voting, ERemote {
  private Hashtable subs = new Hashtable();
  private boolean manyVoteResultArrived = false;
  private boolean exception = false;
  private Object manyVoteResult;
  private String method;
  private int timeout;
  private int listenerCount = 0;
  private Vector listeners = new Vector();
  private HashSet receivedEvents = new HashSet();

  /**
   * Constructor: entry point to the stub class
   * @param env Properties Environment properties
   * @throws RemoteException If something goes wrong ;-)
   */
  public VotingStub (Properties env) throws RemoteException {
    super (env);
    String muid = env.getProperty (Context.CLASS_ID);
    uid = muid;
    className = this.getClass().getName().replaceAll ("Stub", "");
    subs.put ("OBJ", uid + ":RESULT");
    try {
      conn.subscribe (this, subs);
    } catch (SessionException e) {
      throw new ConnectionException (e.getMessage());
    }
  }

  /**
   * Constructor: entry point to the stub class
   * @param env Properties Environment properties
   * @param stub String Stub name
   * @throws RemoteException If something goes wrong ;-)
   */
  public VotingStub (Properties env, String stub) throws RemoteException {
    super (env);
    String muid = env.getProperty (Context.CLASS_ID);
    uid = muid;
    className = this.getClass().getName().replaceAll ("Stub", "");
    subs.put ("OBJ", uid + ":RESULT");
  }

  /**
   * Method invoked each time the stub receives an event
   * @param evt Hashtable Information about the received event
   */
  public void eventArrived (Hashtable evt) {
    method = (String)evt.get ("METHOD");
    if (!receivedEvents.contains (method + (String) evt.get ("TIMESTAMP"))) {
      String type;
      if (method.equals ("manyVote")) {
        manyVoteResult = evt.get ("object");
        type = (String) evt.get ("TYPE");
        if (type.equals ("EXCEPTION")) {
          exception = true;
        }
        manyVoteResultArrived = true;
      }
    }
    receivedEvents.add (method + (String) evt.get ("TIMESTAMP"));
  }

  /**
   * Generated stub manycall method: manyVote
   * @param p0 Integer Input parameter 0
   * @param p1 Integer Input parameter 1
   * @param p2 String Input parameter 2
   * @throws RemoteException If something goes wrong ;-)
   * @return Integer Result
   */
  public Integer manyVote (Integer p0, Integer p1, String p2) throws RemoteException {
    // Set timestamp for this event and its possible retransmissions
    String tstamp = "" + System.currentTimeMillis();

    for (int retransmission = 0; retransmission < Context.RETRANSMISSIONS;) {
      Hashtable evt = new Hashtable();
      evt.put ("OBJECT", uid);
      evt.put ("TYPE", "DISPATCH");
      evt.put ("METHOD", "manyVote");
      evt.put ("TIMESTAMP", tstamp);
      evt.put ("SOURCE_NHANDLE", conn.getLocalNodeHandle());
      manyVoteResultArrived = false;
      Vector args = new Vector();
      args.add (p0);
      args.add (p1);
      args.add (p2);
      evt.put ("object", args);
      try {
        conn.anycall (evt);
      }
      catch (SessionException ex1) {
        throw new RemoteException (ex1.getMessage());
      }
      timeout = 0;
      while (!manyVoteResultArrived && timeout < Context.TIMEOUT) {
        try {
          Thread.currentThread().sleep (Context.DELAY);
        } catch (InterruptedException ex) {
          ex.printStackTrace();
        }
        timeout++;
      }
      if (exception) {
        exception = false;
        RemoteException ex = (RemoteException) manyVoteResult;
        throw ex;
      } else if (timeout == Context.TIMEOUT) {
        retransmission++;
        continue;
      }
      break;
    }
    if (timeout == Context.TIMEOUT) {
      throw new TimeoutException ("Object seems to be down.");
    }
    else {
      Integer res = (Integer) manyVoteResult;
      return res;
    }
  }

  /**
   * Generated stub manycall local condition method: manyVoteCondition
   * @param p0 Integer Input parameter 0
   * @param p1 Integer Input parameter 1
   * @param p2 String Input parameter 2
   * @throws RemoteException If something goes wrong ;-)
   * @return boolean true if manycall's local condition is met
   */
  public boolean manyVoteCondition (Integer p0, Integer p1, String p2) throws RemoteException {
    return manyVoteCondition (p0,p1, p2);
  }

  /**
   * Generated stub manycall global condition method: manyVoteGlobalCondition
   * @param p0 Integer Input parameter 0
   * @param p1 Integer Input parameter 1
   * @param p2 String Input parameter 2
   * @throws RemoteException If something goes wrong ;-)
   * @return boolean true if manycall's global condition is met
   */
  public boolean manyVoteGlobalCondition (Integer p0, Integer p1, String p2) throws RemoteException {
    return manyVoteGlobalCondition (p0, p1, p2);
  }

  /**
   * Closes the stub
   * @throws RemoteException If something goes wrong ;-)
   */
  public void close() throws RemoteException {
    try {
      conn.unsubscribe (this, subs);
      dermi.util.Bus.close();
    } catch (SessionException e) {
      throw new ConnectionException(e.getMessage());
    }
  }

  /**
   * Initializes the stub
   * @throws RemoteException If something goes wrong ;-)
   */
  public void init() throws RemoteException {
    conn = dermi.util.Bus.getBus (eref);
    try {
      conn.subscribe (this, subs);
    } catch (SessionException e) {
      throw new ConnectionException (e.getMessage());
    }
  }

  /**
   * Writes the current object to a stream
   * @param out ObjectOutputStream Data stream
   * @throws IOException  If something goes wrong ;-)
   */
  private void writeObject (ObjectOutputStream out) throws IOException {
    out.defaultWriteObject();
  }

  /**
   * Reads an object from a stream
   * @param in ObjectInputStream Data stream
   * @throws IOException  If something goes wrong ;-)
   * @throws ClassNotFoundException  If something goes wrong ;-)
   */
  private void readObject (ObjectInputStream in) throws IOException, ClassNotFoundException {
    try {
      in.defaultReadObject();
      init();
    } catch (RemoteException e) {
      e.printStackTrace();
    }
  }

  /**
   * Compares this object with another
   * @param obj Object The other object
   * @return boolean true if both equal
   */
  public boolean equals (Object obj) {
    ERemote ref = (ERemote) obj;
    return ref.getERef().equals (getERef());
  }

  /**
   * Returns a stub copy of this class
   * @throws RemoteException If something goes wrong ;-)
   * @return ERemote The returned stub
   */
  public ERemote copy() throws RemoteException {
    ERemote ref = FileRegistry.loadStub (getERef());
    return (ERemote) ref;
  }
}
